@@ -84,5 +84,11 @@ def reload_mappers(metadata):
     import karesansui.db.model.machine2jobgroup
     karesansui.db.model.machine2jobgroup.reload_mapper(metadata, _now)
 
+    import karesansui.db.model.watch
+    karesansui.db.model.watch.reload_mapper(metadata, _now)
+
+    import karesansui.db.model.option
+    karesansui.db.model.option.reload_mapper(metadata, _now)
+
 if __name__ == '__main__':
     pass
