@@ -10,8 +10,13 @@
 # of the License, or (at your option) any later version.
 #
 
-import web
 import re
+import pwd
+import os.path
+import glob
+import time
+
+import web
 
 import karesansui
 from karesansui.lib.rest import Rest, auth
@@ -30,7 +35,12 @@ from karesansui.lib.utils import comma_split, \
      uni_force, get_partition_info, chk_create_disk, json_dumps, \
      get_ifconfig_info, get_keymaps, available_virt_mechs, \
      available_virt_uris, is_iso9660_filesystem_format, \
-     get_dom_list, get_dom_type
+     get_dom_list, get_dom_type, base64_encode
+
+from karesansui.lib.utils import get_xml_parse as XMLParse
+from karesansui.lib.utils import get_xml_xpath as XMLXpath
+from karesansui.lib.utils import get_nums_xml_xpath as XMLXpathNum
+from karesansui.lib.file.configfile import ConfigFile
 
 from karesansui.lib.checker import Checker, \
     CHECK_EMPTY, CHECK_VALID, CHECK_LENGTH, CHECK_ONLYSPACE, \
@@ -44,9 +54,14 @@ from karesansui.lib.const import \
     HYPERVISOR_MIN_SIZE, HYPERVISOR_MAX_SIZE, \
     MEMORY_MIN_SIZE, DISK_MIN_SIZE, \
     DOMAIN_NAME_MIN_LENGTH, DOMAIN_NAME_MAX_LENGTH, \
-    MACHINE_HYPERVISOR, MACHINE_ATTRIBUTE
+    MACHINE_HYPERVISOR, MACHINE_ATTRIBUTE, \
+    DISK_QEMU_FORMAT, DISK_NON_QEMU_FORMAT, \
+    VIRT_COMMAND_CREATE_STORAGE_VOLUME, \
+    VIRT_COMMAND_DELETE_STORAGE_VOLUME, STORAGE_VOLUME_PWD, \
+    DISK_USES
 
 from karesansui.lib.virt.virt import KaresansuiVirtConnection
+from karesansui.lib.virt.config_export import ExportConfigParam
 
 from karesansui.lib.merge import  MergeGuest
 
@@ -57,7 +72,7 @@ from karesansui.db.access.machine import \
 
 from karesansui.db.access.machine2jobgroup import new as m2j_new, save as m2j_save
 from karesansui.db.access.notebook import new as n_new
-from karesansui.db.access.tag import new as t_new, samecount as t_count, findby1name as t_name
+#from karesansui.db.access.tag import new as t_new, samecount as t_count, findby1name as t_name
 from karesansui.db.access.tag import new as t_new, samecount as t_count, findby1name as t_name
 from karesansui.db.access._2pysilhouette import jg_save, jg_delete
 from karesansui.db.model._2pysilhouette import Job, JobGroup
@@ -155,14 +170,16 @@ def validates_guest_add(obj):
                 None,
             ) and check
 
-    if is_param(obj.input, 'vm_disk_size'):
-        check = checker.check_number(
-                _('Disk Size (MB)'),
-                obj.input.vm_disk_size,
-                CHECK_VALID | CHECK_MIN | CHECK_EMPTY,
-                DISK_MIN_SIZE,
-                None,
-            ) and check
+    if is_param(obj.input, 'pool_type'):
+        if obj.input.pool_type != "block":
+            if is_param(obj.input, 'vm_disk_size'):
+                check = checker.check_number(
+                        _('Disk Size (MB)'),
+                        obj.input.vm_disk_size,
+                        CHECK_VALID | CHECK_MIN | CHECK_EMPTY,
+                        DISK_MIN_SIZE,
+                        None,
+                ) and check
 
     if not is_param(obj.input, 'boot_image'):
         check = False
@@ -246,8 +263,44 @@ def validates_guest_add(obj):
 
 
 # public method
+def make_storage_volume_job(uuid, name, pool_name, format,
+                            capacity, allocation, unit, order):
+    cmdname = u"Create Storage Volume"
+    cmd = VIRT_COMMAND_CREATE_STORAGE_VOLUME
+
+    options = {}
+    options['volume_name'] = uuid
+    options['name'] = name
+    options['pool_name'] = pool_name
+    options['format'] = format
+    options['capacity'] = capacity
+    options['allocation'] = allocation
+    options['unit'] = unit
+    options['permission_owner'] = pwd.getpwnam(STORAGE_VOLUME_PWD["OWNER"])[2]
+    options['permission_group'] = pwd.getpwnam(STORAGE_VOLUME_PWD["GROUP"])[2]
+    options['permission_mode'] = STORAGE_VOLUME_PWD["MODE"]
+    options['use'] = DISK_USES["IMAGES"]
+
+    _cmd = dict2command(
+        "%s/%s" % (karesansui.config['application.bin.dir'], cmd), options)
+
+    rollback_options = {}
+    #rollback_options["name"] = name
+    rollback_options["name"] = uuid
+    rollback_options["pool_name"] = pool_name
+    rollback_options["use"] = DISK_USES["IMAGES"]
+
+    rollback_cmd = dict2command(
+        "%s/%s" % (karesansui.config['application.bin.dir'], VIRT_COMMAND_DELETE_STORAGE_VOLUME),
+        rollback_options)
+
+    _job = Job('%s command' % cmdname, order, _cmd)
+    # delete_guestコマンドが削除まで担当してくれるので、ここではロールバックコマンドを設定しない。
+    #_job.rollback_command = rollback_cmd
+    return _job
+
 def regist_guest(obj, _guest, icon_filename,
-                  cmd, options, cmdname, rollback_options):
+                  cmd, options, cmdname, rollback_options, is_create=False):
 
     if icon_filename:
         _guest.icon = icon_filename
@@ -265,10 +318,28 @@ def regist_guest(obj, _guest, icon_filename,
         rollback_options)
 
     _jobgroup = JobGroup(cmdname[0], karesansui.sheconf['env.uniqkey'])
-    _job = Job('%s command' % cmdname[1], 0, action_cmd)
+
+    # create volume job
+    order = 0
+    if is_create is True:
+        _volume_job = make_storage_volume_job(options["uuid"],
+                                              options["storage-volume"],
+                                              options["storage-pool"],
+                                              options["disk-format"],
+                                              options["disk-size"],
+                                              options["disk-size"],
+                                              'M',
+                                              order
+                                              )
+
+        order += 1
+        _jobgroup.jobs.append(_volume_job)
+
+
+    _job = Job('%s command' % cmdname[1], order, action_cmd)
     _job.rollback_command = rollback_cmd
     _jobgroup.jobs.append(_job)
-    
+
     # GuestOS INSERT
     try:
         m_save(obj.orm, _guest)
@@ -277,7 +348,7 @@ def regist_guest(obj, _guest, icon_filename,
         obj.logger.error('Failed to register the Guest OS. #1 - guest name=%s' \
                           % _guest.name)
         raise # throw
-    
+
     # JobGroup INSERT
     try:
         jg_save(obj.pysilhouette.orm, _jobgroup)
@@ -286,7 +357,7 @@ def regist_guest(obj, _guest, icon_filename,
         # rollback(machine)
         obj.logger.error('Failed to register the JobGroup. #2 - jobgroup name=%s' \
                           % _jobgroup.name)
-            
+
         try:
             m_delete(obj.orm, _guest)
             obj.orm.commit()
@@ -294,7 +365,7 @@ def regist_guest(obj, _guest, icon_filename,
         except:
             obj.logger.critical('#4 Rollback failed. - guest id=%d' % _guest.id)
             raise
-            
+
         raise # throw
 
     # Machine2JobGroup INSERT
@@ -334,7 +405,7 @@ class Guest(Rest):
         if hasattr(self, "kvc") is True:
             self.kvc.close()
         return ret
-    
+
     @auth
     def _GET(self, *param, **params):
         host_id = self.chk_hostby1(param)
@@ -343,127 +414,202 @@ class Guest(Rest):
         model = findbyhost1(self.orm, host_id)
         uris = available_virt_uris()
 
-        if self.is_mode_input() is True:
+        self.kvc = KaresansuiVirtConnection()
+        try: # libvirt connection scope -->
+            # Storage Pool
+            #inactive_pool = self.kvc.list_inactive_storage_pool()
+            inactive_pool = []
+            active_pool = self.kvc.list_active_storage_pool()
+            pools = inactive_pool + active_pool
+            pools.sort()
 
-            bridge_prefix = {
-                              "XEN":"xenbr",
-                              "KVM":"eth",
-                            }
-            self.view.host_id = host_id
-            self.view.DEFAULT_KEYMAP = DEFAULT_KEYMAP
+            if not pools:
+                return web.badrequest('One can not start a storage pool.')
 
-            self.view.hypervisors = {}
-            self.view.mac_address = {}
-            self.view.keymaps = {}
-            self.view.phydev = {}
-            self.view.virnet = {}
+            # Output .input
+            if self.is_mode_input() is True:
+                self.view.pools = pools
+                pools_info = {}
+                pools_vols_info = {}
+                pools_iscsi_blocks = {}
+                already_vols = []
+                guests = []
 
-            used_ports = {}
-            #import pdb; pdb.set_trace()
+                guests += self.kvc.list_inactive_guest()
+                guests += self.kvc.list_active_guest()
+                for guest in guests:
+                    already_vol = self.kvc.get_storage_volume_bydomain(domain=guest,
+                                                                       image_type=None,
+                                                                       attr='path')
+                    if already_vol:
+                        already_vols += already_vol.keys()
 
-            for k,v in MACHINE_HYPERVISOR.iteritems():
-                if k in available_virt_mechs():
-                    self.view.hypervisors[k] = v
+                for pool in pools:
+                    pool_obj = self.kvc.search_kvn_storage_pools(pool)[0]
+                    if pool_obj.is_active() is True:
+                        pools_info[pool] = pool_obj.get_info()
 
-                    uri = uris[k]
+                        blocks = None
+                        if pools_info[pool]['type'] == 'iscsi':
+                            blocks = self.kvc.get_storage_volume_iscsi_block_bypool(pool)
+                            if blocks:
+                                pools_iscsi_blocks[pool] = []
+                        vols_obj = pool_obj.search_kvn_storage_volumes(self.kvc)
+                        vols_info = {}
 
-                    try:
-                        self.kvc = KaresansuiVirtConnection(uri)
+                        for vol_obj in vols_obj:
+                            vol_name = vol_obj.get_storage_volume_name()
+                            vols_info[vol_name] = vol_obj.get_info()
+                            if blocks:
+                                if vol_name in blocks and vol_name not in already_vols:
+                                    pools_iscsi_blocks[pool].append(vol_obj.get_info())
 
-                        mem_info        = self.kvc.get_mem_info()
-                        active_networks = self.kvc.list_active_network()
-                        used_vnc_ports  = self.kvc.list_used_vnc_port()
+                        pools_vols_info[pool] = vols_info
 
-                        self.kvc.close()
-                        del self.kvc
-                    except:
-                        if hasattr(self, "kvc") is True:
-                            del self.kvc
-                        continue
+                self.view.pools_info = pools_info
+                self.view.pools_vols_info = pools_vols_info
+                self.view.pools_iscsi_blocks = pools_iscsi_blocks
 
-                    self.view.min_mem = MEMORY_MIN_SIZE
-                    self.view.max_mem = mem_info['host_max_mem']
+                bridge_prefix = {
+                    "XEN":"xenbr",
+                    "KVM":"(eth|bondbr)",
+                    }
+                self.view.host_id = host_id
+                self.view.DEFAULT_KEYMAP = DEFAULT_KEYMAP
+                self.view.DISK_NON_QEMU_FORMAT = DISK_NON_QEMU_FORMAT
+                self.view.DISK_QEMU_FORMAT = DISK_QEMU_FORMAT
 
-                    self.view.mac_address[k] = generate_mac_address(k)
-                    self.view.keymaps[k] = eval("get_keymaps(%s_KEYMAP_DIR)" % k)
+                self.view.hypervisors = {}
+                self.view.mac_address = {}
+                self.view.keymaps = {}
+                self.view.phydev = {}
+                self.view.virnet = {}
 
-                    # Physical device
-                    phydev = []
-                    phydev_regex = re.compile(r"%s" % bridge_prefix[k])
-                    for dev,dev_info in get_ifconfig_info().iteritems():
-                        try:
-                            if phydev_regex.match(dev):
-                                phydev.append(dev)
-                        except:
-                            pass
-                    if len(phydev) == 0:
-                        phydev.append("%s0" % bridge_prefix[k])
-                    phydev.sort()
-                    self.view.phydev[k] = phydev # Physical device
-
-                    # Virtual device
-                    self.view.virnet[k] = sorted(active_networks)
-                    used_ports[k] = used_vnc_ports
-
-
-            exclude_ports = []
-            for k, _used_port in used_ports.iteritems():
-                exclude_ports = exclude_ports + _used_port
-                exclude_ports = sorted(exclude_ports)
-                exclude_ports = [p for p, q in zip(exclude_ports, exclude_ports[1:] + [None]) if p != q]
-            self.view.vnc_port = next_number(VNC_PORT_MIN_NUMBER,
-                                             PORT_MAX_NUMBER,
-                                             exclude_ports)
-
-            partition_info = get_partition_info(VIRT_DOMAINS_DIR)
-            disk_info = {"host_max_disk": int(partition_info[1][0]),
-                         "guest_alloc_disk":int(partition_info[2][0]),
-                         "host_free_disk":int(partition_info[3][0]),
-                         }
-
-            self.view.min_disk = DISK_MIN_SIZE
-            self.view.max_disk = disk_info["host_max_disk"]
-
-            return True
-
-        else:
-            models = findbyhost1guestall(self.orm, host_id)
-            if not models: # machine not found.
-                return True
-
-            guests = []
-            self.view.hypervisors = {}
-            for model in models:
+                used_ports = {}
 
                 for k,v in MACHINE_HYPERVISOR.iteritems():
                     if k in available_virt_mechs():
                         self.view.hypervisors[k] = v
                         uri = uris[k]
+                        mem_info = self.kvc.get_mem_info()
+                        active_networks = self.kvc.list_active_network()
+                        used_vnc_ports = self.kvc.list_used_vnc_port()
+                        bus_types = self.kvc.bus_types
+                        self.view.bus_types = bus_types
+                        self.view.max_mem = mem_info['host_max_mem']
+                        self.view.free_mem = mem_info['host_free_mem']
+                        self.view.alloc_mem = mem_info['guest_alloc_mem']
 
-                        try:
-                            if hasattr(self, "kvc") is not True:
-                                self.kvc = KaresansuiVirtConnection(uri)
-                            domname = self.kvc.uuid_to_domname(model.uniq_key)
-                            #if not domname: return web.conflict(web.ctx.path)
-                            _virt = self.kvc.search_kvg_guests(domname)
-                            if len(_virt) > 0:
-                                guests.append(MergeGuest(model, _virt[0]))
+                        self.view.mac_address[k] = generate_mac_address(k)
+                        self.view.keymaps[k] = eval("get_keymaps(%s_KEYMAP_DIR)" % k)
+
+                        # Physical device
+                        phydev = []
+                        phydev_regex = re.compile(r"%s" % bridge_prefix[k])
+                        for dev,dev_info in get_ifconfig_info().iteritems():
+                            try:
+                                if phydev_regex.match(dev):
+                                    phydev.append(dev)
+                            except:
+                                pass
+                        if len(phydev) == 0:
+                            phydev.append("%s0" % bridge_prefix[k])
+                        phydev.sort()
+                        self.view.phydev[k] = phydev # Physical device
+
+                        # Virtual device
+                        self.view.virnet[k] = sorted(active_networks)
+                        used_ports[k] = used_vnc_ports
+
+
+                exclude_ports = []
+                for k, _used_port in used_ports.iteritems():
+                    exclude_ports = exclude_ports + _used_port
+                    exclude_ports = sorted(exclude_ports)
+                    exclude_ports = [p for p, q in zip(exclude_ports, exclude_ports[1:] + [None]) if p != q]
+                self.view.vnc_port = next_number(VNC_PORT_MIN_NUMBER,
+                                                 PORT_MAX_NUMBER,
+                                                 exclude_ports)
+
+            else: # .part
+                models = findbyhost1guestall(self.orm, host_id)
+                guests = []
+                if models:
+                    # Physical Guest Info
+                    self.view.hypervisors = {}
+                    for model in models:
+                        for k,v in MACHINE_HYPERVISOR.iteritems():
+                            if k in available_virt_mechs():
+                                self.view.hypervisors[k] = v
+                                uri = uris[k]
+                                if hasattr(self, "kvc") is not True:
+                                    self.kvc = KaresansuiVirtConnection(uri)
+                                domname = self.kvc.uuid_to_domname(model.uniq_key)
+                                #if not domname: return web.conflict(web.ctx.path)
+                                _virt = self.kvc.search_kvg_guests(domname)
+                                if 0 < len(_virt):
+                                    guests.append(MergeGuest(model, _virt[0]))
+                                else:
+                                    guests.append(MergeGuest(model, None))
+
+                # Exported Guest Info
+                exports = {}
+                for pool_name in pools:
+                    files = []
+
+                    pool = self.kvc.search_kvn_storage_pools(pool_name)
+                    path = pool[0].get_info()["target"]["path"]
+
+                    if os.path.exists(path):
+                        for _afile in glob.glob("%s/*/info.dat" % (path,)):
+                            param = ExportConfigParam()
+                            param.load_xml_config(_afile)
+
+                            _dir = os.path.dirname(_afile)
+
+                            uuid = param.get_uuid()
+                            name = param.get_domain()
+                            created = param.get_created()
+                            title = param.get_title()
+                            if title != "":
+                                title = re.sub("[\r\n]","",title)
+                            if title == "":
+                                title = _('untitled')
+
+                            if created != "":
+                                created_str = time.strftime("%Y/%m/%d %H:%M:%S", \
+                                                            time.localtime(float(created)))
                             else:
-                                guests.append(MergeGuest(model, None))
-                        except:
-                            if hasattr(self, "kvc") is True:
-                                del self.kvc
+                                created_str = _("N/A")
 
-            if self.is_json() is True:
-                guests_json = []
-                for x in guests:
-                    guests_json.append(x.get_json(self.me.languages))
-                        
-                self.view.guests = json_dumps(guests_json)
-            else:
-                self.view.guests = guests
+                            files.append({"dir": _dir,
+                                          "pool" : pool_name,
+                                          #"b64dir" : base64_encode(_dir),
+                                          "uuid" : uuid,
+                                          "name" : name,
+                                          "created" : int(created),
+                                          "created_str" : created_str,
+                                          "title" : title,
+                                          "icon" : param.get_database()["icon"],
+                                          })
+
+                    exports[pool_name] = files
+
+                # .json
+                if self.is_json() is True:
+                    guests_json = []
+                    for x in guests:
+                        guests_json.append(x.get_json(self.me.languages))
+
+                    self.view.guests = json_dumps(guests_json)
+                else:
+                    self.view.exports = exports
+                    self.view.guests = guests
 
             return True
+        finally:
+            #self.kvc.close()
+            pass # libvirt connection scope --> Guest#_post()
 
     @auth
     def _POST(self, *param, **params):
@@ -484,16 +630,30 @@ class Guest(Rest):
             return web.badrequest(self.view.alert)
 
         try:
-            self.kvc = KaresansuiVirtConnection(uri)
-            active_guests = self.kvc.list_active_guest()
-            inactive_guests = self.kvc.list_inactive_guest()
-            used_vnc_ports = self.kvc.list_used_vnc_port()
-            used_mac_addrs = self.kvc.list_used_mac_addr()
-            mem_info = self.kvc.get_mem_info()
+            try:
+                self.kvc = KaresansuiVirtConnection(uri)
+                active_guests = self.kvc.list_active_guest()
+                inactive_guests = self.kvc.list_inactive_guest()
+                used_vnc_ports = self.kvc.list_used_vnc_port()
+                used_mac_addrs = self.kvc.list_used_mac_addr()
+                mem_info = self.kvc.get_mem_info()
+
+                if is_param(self.input, "vm_mem_size"):
+                    if mem_info['host_free_mem'] < int(self.input.vm_mem_size):
+                        return web.badrequest(_("Space not enough to allocate guest memory."))
+
+                if is_param(self.input, "pool_type") and \
+                       self.input.pool_type != "block" and \
+                       is_param(self.input, "pool_dir"):
+                    target_path = self.kvc.get_storage_pool_targetpath(self.input.pool_dir)
+                    if target_path: # disk
+                        if not chk_create_disk(target_path, self.input.vm_disk_size):
+                            partition = get_partition_info(target_path, header=False)
+                            return web.badrequest(_("No space available to create disk image in '%s' partition.") % partition[5][0])
+            except:
+                raise
+        finally:
             del self.kvc
-        except:
-            del self.kvc
-            raise
 
         # Check on whether value has already been used
         # Guest OS
@@ -507,32 +667,44 @@ class Guest(Rest):
         if(self.input.vm_mac in used_mac_addrs):
             return web.conflict(web.ctx.path, "MAC Address is already there.")
 
-        # disk
-        if not chk_create_disk(VIRT_DOMAINS_DIR, self.input.vm_disk_size):
-            return web.badrequest('No available disk size.')
-
         uuid = string_from_uuid(generate_uuid())
-        
+
         options = {}
         options['uuid'] = uuid
-        
+
         if is_param(self.input, "domain_name"):
             options['name'] = self.input.domain_name
         if is_param(self.input, "vm_mem_size"):
             options['mem-size'] = self.input.vm_mem_size
-        if is_param(self.input, "vm_disk_size"):
-            options['disk-size'] = self.input.vm_disk_size
         if is_param(self.input, "vm_kernel"):
             options['kernel'] = self.input.vm_kernel
         if is_param(self.input, "vm_initrd"):
             options['initrd'] = self.input.vm_initrd
         if is_param(self.input, "vm_iso"):
             options['iso'] = self.input.vm_iso
-
         if is_param(self.input, "keymap"):
             options['keymap'] = self.input.keymap
-        if is_param(self.input, "sparse") and self.input.sparse == "1":
-            options['sparse'] = None
+
+        is_create = False
+        if is_param(self.input, "pool_type"):
+            if is_param(self.input, "bus_type"):
+                options['bus'] = self.input.bus_type
+
+            if self.input.pool_type == "dir" or self.input.pool_type == "fs": # create volume
+                is_create = True
+                options['disk-format'] = self.input.disk_format
+                options["storage-pool"] = self.input.pool_dir
+                options["storage-volume"] = options['name'] # default domain name
+                options['disk-size'] = self.input.vm_disk_size
+
+            elif self.input.pool_type == "block": # iscsi block device
+                (iscsi_pool, iscsi_volume) = self.input.pool_dir.split("/", 2)
+                options["storage-pool"] = iscsi_pool
+                options["storage-volume"] = iscsi_volume
+            else:
+                return web.badrequest()
+        else:
+            return web.badrequest()
 
         if is_param(self.input, "vm_vncport"):
             options['vnc-port'] = self.input.vm_vncport
@@ -565,7 +737,7 @@ class Guest(Rest):
         note_value = None
         if is_param(self.input, "note_value"):
             note_value = self.input.note_value
-            
+
         _notebook = n_new(note_title, note_value)
 
         # tags
@@ -584,7 +756,7 @@ class Guest(Rest):
         icon_filename = None
         if is_param(self.input, "icon_filename", empty=True):
             icon_filename = self.input.icon_filename
-            
+
         _guest = m_new(created_user=self.me,
                        modified_user=self.me,
                        uniq_key=uni_force(uuid),
@@ -599,13 +771,17 @@ class Guest(Rest):
                        )
 
         ret =  regist_guest(self,
-                             _guest,
-                             icon_filename,
-                             VIRT_COMMAND_CREATE_GUEST,
-                             options,
-                             ['Create Guest', 'Create Guest'],
-                             {"name": options['name']}
-                             )
+                            _guest,
+                            icon_filename,
+                            VIRT_COMMAND_CREATE_GUEST,
+                            options,
+                            ('Create Guest', 'Create Guest'),
+                            {"name": options['name'],
+                             "pool" : options["storage-pool"],
+                             "volume" : options["uuid"],
+                             },
+                            is_create,
+                            )
         if ret is True:
             return web.accepted()
         else:
