@@ -453,7 +453,7 @@ class Checker(object):
 
         if check & CHECK_EMPTY:
             ret_val = self.check_empty(name, value)
- 
+
         if ret_val and check & CHECK_LENGTH:
             ret_val = self.check_length(name, value, min, max)
 
@@ -650,7 +650,7 @@ class Checker(object):
 
         if check & CHECK_EMPTY:
             ret_val = self.check_empty(name, value)
- 
+
         if ret_val and (check & CHECK_VALID) and len(value) > 0:
             regex = '^([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}$'
             m = re.compile(regex).search(value)
@@ -660,6 +660,82 @@ class Checker(object):
 
         return ret_val
 
+    def check_netmask(self, name, value, check):
+        """<comment-ja>
+        指定したネットマスクが正しいフォーマットであるかどうかをチェックする
+
+        @param name: 入力項目名
+        @param value: チェックするネットマスク
+        @param check: チェックする条件
+        @return: チェック結果
+        @rtype: boolean
+        </comment-ja>
+        <comment-en>
+        Determine whether netmask is valid format.
+
+        @param name: Item name
+        @param value: netmask
+        @param check: condition to determine
+        @return: result
+        @rtype: boolean
+        </comment-en>
+        """
+        ret_val = True;
+
+        if isinstance(value, int):
+            value = str(value)
+
+        if check & CHECK_EMPTY:
+            ret_val = self.check_empty(name, value)
+
+        if value and ret_val:
+            if check & CHECK_VALID:
+                from karesansui.lib.networkaddress import NetworkAddress
+
+                if not NetworkAddress().valid_netmask(value):
+                    ret_val = False
+                    self.add_error(_('%s is in invalid format.') % (name,))
+
+        return ret_val
+
+    def check_cidr(self, name, value, check):
+        """<comment-ja>
+        指定したネットワークアドレス（CIDR）が正しいフォーマットであるかどうかをチェックする
+
+        @param name: 入力項目名
+        @param value: チェックするネットワークアドレス
+        @param check: チェックする条件
+        @return: チェック結果
+        @rtype: boolean
+        </comment-ja>
+        <comment-en>
+        Determine whether cidr-style network address is valid format.
+
+        @param name: Item name
+        @param value: network address
+        @param check: condition to determine
+        @return: result
+        @rtype: boolean
+        </comment-en>
+        """
+        ret_val = True;
+
+        if isinstance(value, int):
+            value = str(value)
+
+        if check & CHECK_EMPTY:
+            ret_val = self.check_empty(name, value)
+
+        if value and ret_val:
+            if check & CHECK_VALID:
+                from karesansui.lib.networkaddress import NetworkAddress
+
+                if not NetworkAddress(value).valid_addr():
+                    ret_val = False
+                    self.add_error(_('%s is in invalid format.') % (name,))
+
+        return ret_val
+
     def check_netdev_name(self, name, value, check):
         """<comment-ja>
         指定したネットワークデバイス名が正しいかどうかをチェックする
@@ -726,7 +802,11 @@ class Checker(object):
             ret_val = self.check_empty(name, value)
 
         if ret_val and (check & CHECK_VALID):
-            regex = "^[a-zA-Z0-9][a-zA-Z0-9 \.\:]{0,24}$" # what is a valid network name for libvirt?
+            # Depend on maximum length of pid file name for dnsmasq.
+            # len("<network_name>.pid") <= 256
+            # libvirt allow space or other special chars,
+            # but virsh command cannot parse space-included name.
+            regex = "^[a-zA-Z0-9][a-zA-Z0-9\_\.\:\-]{0,251}$"
             m = re.compile(regex).search(value)
             if not m:
                 ret_val = False
@@ -1312,7 +1392,7 @@ class Checker(object):
                         info = network.get_info()
                         existing_na_cidr = '%s/%s' % (info['ip']['address'], info['ip']['netmask'])
                         existing_na = NetworkAddress(existing_na_cidr)
-                        
+
                         conflict = False
                         # New-one in existing-one
                         if existing_na.network_includes_address(new_na.network): conflict = True
@@ -1532,3 +1612,105 @@ class Checker(object):
                 self.add_error(_('No such %s [%s].') % (name, value))
 
         return ret_val
+
+    def check_fraction(self, name, value, check, min=None, max=None, precision=None):
+        """<comment-ja>
+        指定した実数値が正しいものであるかどうかをチェックする
+
+        @param name: 入力項目名
+        @param value: チェックする実数値
+        @param check: チェックする条件
+        @param min: 最小値
+        @param max: 最大値
+        @param precision: 小数点以下の桁数
+        @return: チェック結果
+        @rtype: boolean
+        </comment-ja>
+        <comment-en>
+        Determine whether fraction is valid format or in specified range.
+
+        @param name: Item name
+        @param value: interger
+        @param check: condition to determine
+        @param min: the minimum length
+        @param max: the maximum length
+        @param precision: Number of decimal places
+        @return: result
+        @rtype: boolean
+        </comment-en>
+        """
+        ret_val = True;
+
+        if isinstance(value, int):
+            value = str(value)
+
+        if check & CHECK_EMPTY:
+            ret_val = self.check_empty(name, value)
+
+        if value!="" and ret_val:
+            arr_dp = re.compile("^([-+]?[0-9]+)\.([0-9]+)$").match(value)
+
+            if arr_dp:
+                integer = arr_dp.group(1)
+                fraction = arr_dp.group(2)
+            else:
+                integer = value
+                fraction = "0"
+
+            if check & CHECK_VALID:
+                if not re.compile("^[-+]?[0-9]+$").match(integer) or not re.compile("^[0-9]+$").match(fraction):
+                    ret_val = False
+                    self.add_error(_('%s is not integer or decimal fraction.') % (name,))
+                elif len(fraction) > precision:
+                    ret_val = False
+                    self.add_error(_('(%s) Length of figures after point must be equal or less than %s.') % (name,str(precision)))
+
+            if ret_val and (check & CHECK_MIN):
+                value = float(value)
+                if value < min:
+                    ret_val = False
+                    self.add_error(_('%s must be greater than %d.') % (name, min))
+
+            if ret_val and (check & CHECK_MAX):
+                value = float(value)
+                if value > max:
+                    ret_val = False
+                    self.add_error(_('%s must be smaller than %d.') % (name, max))
+
+        return ret_val
+
+    def check_time_string(self, name, value, check):
+        """<comment-ja>
+        時刻を示す文字列が正しいかどうかをチェックする
+
+        @param name: 入力項目名
+        @param value: 時刻を示す文字列
+        @param check: チェックする条件
+        @return: チェック結果
+        @rtype: boolean
+        </comment-ja>
+        <comment-en>
+        Determine whether time string is valid format.
+
+        @param name: Item name
+        @param value: string of datetime
+        @param check: condition to determine
+        @return: result
+        @rtype: boolean
+        </comment-en>
+        """
+        ret_val = True
+
+        if is_int(value):
+            value = str(value)
+
+        if check & CHECK_EMPTY:
+            ret_val = self.check_empty(name, value)
+
+        if check & CHECK_VALID:
+            time_regex = re.compile("^([0-1][0-9]|[2][0-3]|[0-9]):[0-5][0-9]$")
+            if not time_regex.match(value):
+                ret_val = False
+                self.add_error(_('%s is invalid format.') % (name,))
+
+        return ret_val
