@@ -43,18 +43,22 @@ import math
 import glob
 import fcntl
 
-from xml.dom import minidom
-from xml import xpath
-import simplejson as json
+import karesansui
+import karesansui.lib.locale
 
 from karesansui import KaresansuiLibException
 from karesansui.lib.const import CHECK_DISK_QUOTA
 from karesansui.lib.networkaddress import NetworkAddress
 
-def preprint_r(var,indent=2,depth=None):
+def preprint_r(var,indent=2,depth=None,return_var=False):
     import pprint
+
     pp = pprint.PrettyPrinter(indent=indent,depth=depth)
-    pp.pprint(var)
+    if return_var is True:
+        return pp.pformat(var)
+    else:
+        pp.pprint(var)
+        return True
 
 def dotsplit(val):
     """<comment-ja>
@@ -111,6 +115,7 @@ def uniq_sort(array):
     </comment-en>
     """
     array = sorted(array)
+    #array = [x for i,x in enumerate(array) if array.index(x) == i]
     array = [_x for _x, _y in zip(array, array[1:] + [None]) if _x != _y]
     return array
 
@@ -183,6 +188,64 @@ def oct2dec(string):
     """
     return int(string, 8)
 
+def float_from_string(string):
+    """<comment-ja>
+    浮動小数点表記の文字列を数値に変換する
+    @param string: 浮動小数点表記
+    @rtype: float
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+
+    >>> from karesansui.lib.utils import float_from_string
+    >>> 
+    >>> float_from_string("9.95287e+07")        # 99528700
+    99528700.0
+    >>> float_from_string("2.07499e+08")        # 207499000
+    207499000.0
+    >>> float_from_string("-2.07499e+02")       # -207.499
+    -207.499
+    >>> float_from_string("+2.07499e+02")       # 207.499
+    207.499
+    >>> float_from_string("+2.07499e-02")       # 0.0207499
+    0.0207499
+    >>> float_from_string("3.39112632978e+001") # 3391126329780.0
+    33.9112632978
+    """
+
+    if type(string) != str:
+        return False
+
+    regex = re.compile("^([\+\-]?)(([0-9]+|[0-9]*[\.][0-9]+))([eE])([\+\-]?)([0-9]+)$")
+    m = regex.match(string)
+    if m:
+        sign = m.group(1)
+        mantissa = float(m.group(2))
+        sign_exponent = m.group(5)
+        exponent = int(m.group(6))
+
+        data = float(1)
+
+        if sign == "-":
+            data = data * -1
+
+        if sign_exponent == "-":
+            exponent = -1 * exponent
+
+        data = data * mantissa * (10**exponent)
+
+    else:
+        return False
+
+    return data
+
+def ucfirst(string):
+    return string[0].upper() + string[1:]
+
+def lcfirst(string):
+    return string[0].lower() + string[1:]
+
 def next_number(min,max,exclude_numbers):
     """
     <comment-ja>
@@ -205,6 +268,52 @@ def next_number(min,max,exclude_numbers):
       if not _x in exclude_numbers:
         return _x
 
+def isset(string, vars=globals(), verbose=False):
+    """
+    bool isset(str string  [,bool verbose=False])
+    <comment-ja>
+    変数名がセットされていることを検査する
+    
+    @param string: 変数名を示す文字列
+    @type string: str
+    @param vars: 検査対象の変数配列
+    @type vars: dict
+    @param verbose: エラーメッセージを表示する場合はTrue
+    @type verbose: bool
+    @return: 変数名がセットされていればTrue、いなければFalse
+    @rtype: bool
+    </comment-ja>
+    <comment-en>
+    Determine if a variable is set.
+
+    @param string: The variable name, as a string. 
+    @type string: str
+    @param vars: all variables available in scope
+    @type vars: dict
+    @param verbose: If used and set to True, isset() will output error messages.
+    @type verbose: bool
+    @return: Returns True if a variable is set, False otherwise. 
+    @rtype: bool
+    </comment-en>
+    """
+    retval = False
+    try:
+        for _k,_v in vars.iteritems():
+            exec("%s = _v" % _k)
+        exec("%s" % string)
+        retval = True
+    except NameError, e:
+        if verbose is True:
+            print _("Notice: Undefined variable \"%s\"") % str(e.args)
+    except KeyError, e:
+        if verbose is True:
+            print _("Notice: Undefined index \"%s\"") % str(e.args)
+    except Exception, e:
+        if verbose is True:
+            print _("Notice: Undefined variable \"%s\"") % str(e.args)
+        pass
+    return retval
+
 def is_uuid(uuid=None):
     """<comment-ja>
     KaresansuiのUUIDフォーマットに対応しているか。
@@ -360,34 +469,56 @@ def detect_encoding(string,encoding_list=None):
 
     return None
 
-def execute_command(command_args):
+def execute_command(command_args,user=None,env=None):
     """
     <comment-ja>
     コマンドを実行する
 
     @param command_args: 実行するコマンドとその引数を要素とする配列
+    @param user: 実行するユーザー名 (省略時はgetuidによる)
+    @param env:  実行時に適用する環境変数の辞書配列
     @return: 終了ステータスと実行時の出力結果の配列
     </comment-ja>
     <comment-en>
     Execute command
 
     @param command_args: The array that consists of command name and its arguments.
+    @param user: the effective user id 
+    @param env:  dictionary of environ variables
     @return: The return status of the executed command
     </comment-en>
     """
     ret = -1
     res = []
 
+    curdir  = os.getcwd()
+
+    if user is None:
+        homedir = pwd.getpwuid(os.getuid())[5]
+    else:
+        try:
+            int(user)
+            homedir = pwd.getpwuid(int(user))[5]
+        except:
+            try:
+                homedir = pwd.getpwnam(user)[5]
+            except:
+                homedir = pwd.getpwuid(os.getuid())[5]
+
     subproc_args = { 'stdin': subprocess.PIPE,
                      'stdout': subprocess.PIPE,
                      'stderr': subprocess.STDOUT,
 #                     'shell': True,
-                     'cwd': os.getcwd(),
+                     'cwd': homedir,
                      'close_fds': True,
                    }
+
+    if env is not None:
+        subproc_args['env'] = env
+
     try:
         pp = subprocess.Popen(command_args, **subproc_args)
-    except OSError:
+    except OSError, e:
         #raise "Failed to execute command: %s" % command_args[0]
         return [ret,res]
 
@@ -397,11 +528,50 @@ def execute_command(command_args):
         if not line:
             break
         line = line.rstrip()
-        res.append(unicode(line, detect_encoding(line)).encode("utf-8"))
-    ret = pp.wait()
+
+        try:
+            res.append(unicode(line, detect_encoding(line)).encode("utf-8"))
+        except:
+            res.append(line)
+
+    try:
+        ret = pp.wait()
+    except:
+        ret = 0
+
     return [ret,res]
 
 
+def pipe_execute_command(commands):
+    """
+    <comment-ja>
+    PIPE付きコマンドを実行する
+
+    @param commands: パイプ付きのコマンドを分割した、配列  example) [['rpm', '-qa'], ['grep', 'karesansui'],]
+    @return: 終了ステータスと実行時の出力結果の配列
+    </comment-ja>
+    <comment-en>
+    </comment-en>
+    """
+    ret = -1
+    res = []
+    out = []
+    for cmd in commands:
+        subproc_args = {'stdin': subprocess.PIPE,
+                        'stdout': subprocess.PIPE,
+                        }
+        if 0 < len(out):
+            subproc_args['stdin'] = out[len(out)-1]
+
+        pp = subprocess.Popen(cmd, **subproc_args)
+
+        (stdout, stdin) = (pp.stdout, pp.stdin)
+        out.append(stdout)
+        ret = pp.wait()
+    res = out[len(out)-1].read()
+    return (ret,res)
+
+
 def create_file(file, value) :
     """
     <comment-ja>
@@ -525,7 +695,9 @@ def create_qemu_disk_img(file,size,type="raw") :
     @return: None
     </comment-en>
     """
-    available_formats = [ "raw","qcow","qcow2","cow","vmdk","cloop" ]
+    from karesansui.lib.const import DISK_QEMU_FORMAT
+    #available_formats = [ "raw","qcow","qcow2","cow","vmdk","cloop" ]
+    available_formats = DISK_QEMU_FORMAT.values()
 
     if type in available_formats:
         command_args = [
@@ -563,7 +735,9 @@ def create_disk_img(file,size,type="raw",is_sparse=True) :
     @return: None
     </comment-en>
     """
-    available_formats = [ "raw","qcow","qcow2","cow","vmdk","cloop" ]
+    from karesansui.lib.const import DISK_QEMU_FORMAT
+    #available_formats = [ "raw","qcow","qcow2","cow","vmdk","cloop" ]
+    available_formats = DISK_QEMU_FORMAT.values()
 
     if type in available_formats:
         if type == "raw":
@@ -604,7 +778,7 @@ def copy_file(src_file,dest_file):
             if not os.path.exists(dest_path):
                 os.makedirs(dest_path)
 
-            (ret, res) = execute_command(["cp",src_file,dest_file])
+            (ret, res) = execute_command(["cp","-rfp",src_file,dest_file])
             if ret == 0:
                 ret = True
         except:
@@ -615,7 +789,143 @@ def copy_file(src_file,dest_file):
     return ret
 
 
+def copy_file_cb(src,dst,callback=None,sparse=False,each=True):
+    default_block_size = 4096
+
+    if type(src) == str:
+        src = [src]
+
+    if type(dst) == str:
+        dst = [dst]
+
+    if len(src) != len(dst):
+        raise
+
+    if each is not True:
+        all_bytes = 0
+        for _src in src:
+            all_bytes = all_bytes + os.path.getsize(_src)
+
+        text = "Copying"
+        callback.start(filename=None, size=long(all_bytes), text=text)
+        cnt = 0
+
+    i = 0
+    all_bytes = 0
+    for _src in src:
+        try:
+            _dst = str(dst[i])
+        except:
+            _dst = dst[i]
+
+        if os.path.exists(os.path.dirname(_dst)) is False:
+            os.makedirs(os.path.dirname(_dst))
+
+        bytes = os.path.getsize(_src)
+        all_bytes = all_bytes + bytes
+
+        if each is True:
+            text = _("Copying %s") % os.path.basename(_src)
+            callback.start(filename=_src, size=long(bytes), text=text)
+            cnt = 0
+
+
+        if os.path.exists(_dst) is False and sparse is True:
+            block_size = default_block_size
+            fd = None
+            try:
+                fd = os.open(_dst, os.O_WRONLY|os.O_CREAT)
+                os.ftruncate(fd, bytes)
+            finally:
+                if fd:
+                    os.close(fd)
+        else:
+            block_size = 1024*1024*10
+
+        #nulls = '\0' * default_block_size
+        nulls = 0x00 * default_block_size
+
+        src_fd = None
+        dst_fd = None
+        try:
+            try:
+                src_fd = os.open(_src, os.O_RDONLY)
+                dst_fd = os.open(_dst, os.O_WRONLY|os.O_CREAT)
+
+                while 1:
+                    data = os.read(src_fd, block_size)
+                    sz = len(data)
+                    if sz == 0:
+                        if each is True:
+                            callback.end(bytes)
+                        else:
+                            callback.update(all_bytes)
+                        break
+                    if sparse and nulls == data:
+                        os.lseek(dst_fd, sz, 1)
+                    else:
+                        b = os.write(dst_fd, data)
+                        if sz != b:
+                            if each is True:
+                                callback.end(cnt)
+                            else:
+                                callback.update(cnt)
+                            break
+                    cnt += sz
+                    if cnt < bytes:
+                        callback.update(cnt)
+            except OSError, e:
+                raise RuntimeError(_("ERROR: copying %s to %s: %s") % (_src,_dst,str(e)))
+        finally:
+            if src_fd is not None:
+                os.close(src_fd)
+            if dst_fd is not None:
+                os.close(dst_fd)
+
+        i = i + 1
+    callback.end(all_bytes)
+
+def move_file(src_file,dest_file):
+    """
+    <comment-ja>
+    ファイルを移動する
+
+    @param src_file: 移動元ファイル名
+    @param dest_file: 移動先ファイル名
+    @return: 移動先先ファイル名
+    </comment-ja>
+    <comment-en>
+    Copy file
+
+    @param src_file: Path to the source file
+    @param dest_file: The destination path
+    @return: The destination path
+    </comment-en>
+    """
+    #ret = shutil.copy2(src_file, dest_file)
+    ret = False
+    if os.path.exists(src_file):
+        try:
+            if dest_file[0] != "/":
+                dest_path = "%s/%s" % (os.getcwd(),os.path.dirname(dest_file),)
+            else:
+                dest_path = os.path.dirname(dest_file)
+            if not os.path.exists(dest_path):
+                os.makedirs(dest_path)
+
+            (ret, res) = execute_command(["mv","-f",src_file,dest_file])
+            if ret == 0:
+                ret = True
+        except:
+            return False
+    else:
+        return False
+
+    return ret
+
 def get_xml_parse(file):
+    from xml.dom import minidom
+
     if os.path.exists(file):
         document = minidom.parse(file)
     else:
@@ -640,6 +950,8 @@ def get_xml_xpath(document, expression):
     @return: Returns node list containing all nodes matching the given XPath expression
     </comment-en>
     """
+    from xml import xpath
+
     result = None
     for i in xpath.Evaluate(expression, document.documentElement):
         result = i.nodeValue
@@ -663,6 +975,8 @@ def get_nums_xml_xpath(document, expression):
     @return: Returns the number of node containing all nodes matching the given XPath expression
     </comment-en>
     """
+    from xml import xpath
+
     result = None
     return xpath.Evaluate('count(%s)' % expression, document.documentElement)
 
@@ -1185,6 +1499,7 @@ def get_model_name(model):
 def chk_create_disk(target, disk_size):
     """<comment-ja>
     指定されたフォルダ/ファイルが属するパーティションに、指定したサイズのファイルが作成できるか。
+    比較単位はMB
 
     @param target: 調べるファイル/フォルダパス
     @type target: str 
@@ -1475,6 +1790,48 @@ def get_proc_cpuinfo(path="/proc/cpuinfo"):
     except:
         return None
 
+def get_process_id(command=None,regex=False):
+    """
+    <comment-ja>
+    指定した文字列のプロセスIDを取得する
+
+    @param command: コマンド文字列
+    @type command: str
+    @param regex: 正規表現による指定かどうか
+    @type regex: boolean
+    @return: プロセスIDのリスト
+    @rtype: list
+    </comment-ja>
+    <comment-en>
+    English Comment
+    </comment-en>
+
+    >>> from ps_match import get_process_id
+    >>> get_process_id("hde-libvirtd",regex=True)
+    ['26859']
+    >>> get_process_id("/opt/hde/sbin/libvirtd --daemon --config /etc/opt/hde/libvirt/libvirtd.conf --listen --pid-file=/var/run/hde-libvirtd.pid",regex=False)
+    ['26859']
+    """
+
+    retval = []
+
+    proc_dir = "/proc"
+    cmdline_file_glob = "%s/[0-9]*/cmdline" % (proc_dir,)
+    for _file in glob.glob(cmdline_file_glob):
+        try:
+            data = open(_file).read()
+            data = re.sub("\0"," ",data)
+            pid = os.path.basename(os.path.dirname(_file))
+            if regex is False:
+                if data.strip() == command:
+                    retval.append(pid)
+            else:
+                if re.search(command,data):
+                    retval.append(pid)
+        except:
+            pass
+    return retval
+
 def json_dumps(obj, **kw):
     """
     <comment-ja>
@@ -1491,6 +1848,7 @@ def json_dumps(obj, **kw):
     English Comment
     </comment-en>
     """
+    import simplejson as json
     return json.dumps(obj, ensure_ascii=False, encoding="utf-8", **kw)
 
 def is_path(target):
@@ -1959,16 +2317,19 @@ def get_disk_img_info(file):
         regex_bracket = re.compile(r"""^(?P<unitformat>.+) \((?P<bytes>[0-9\.]+) bytes\)""")
         ret = {}
         for line in res:
-            key, value = regex.split(line,1)
-            if key == "file format":
-                ret["file_format"] = value
-            elif key == "virtual size":
-                m = regex_bracket.match(value)
-                ret["virtual_size"] = int(m.group("bytes"))
-            elif key == "disk size":
-                ret["disk_size"] = sizeunit_to_byte(value)
-            elif key == "cluster_size":
-                ret["cluster_size"] = int(value)
+            try:
+                key, value = regex.split(line,1)
+                if key == "file format":
+                    ret["file_format"] = value
+                elif key == "virtual size":
+                    m = regex_bracket.match(value)
+                    ret["virtual_size"] = int(m.group("bytes"))
+                elif key == "disk size":
+                    ret["disk_size"] = sizeunit_to_byte(value)
+                elif key == "cluster_size":
+                    ret["cluster_size"] = int(value)
+            except:
+                pass 
         ret["real_size"] = os.path.getsize(file)
         return ret
 
@@ -2078,3 +2439,534 @@ def get_dom_type(domain):
         doc = get_xml_parse(_path)
         retval = get_xml_xpath(doc,'/domain/@type')
     return retval
+
+def base64_encode(string=""):
+    import base64
+
+    if type(string) == unicode:
+        string = string.encode("utf-8")
+    elif type(string) == str:
+        pass
+    else:
+        raise
+
+    return base64.b64encode(string)
+
+def base64_decode(string=""):
+    import base64
+
+    if type(string) == str:
+        pass
+    else:
+        raise
+
+    return base64.b64decode(string)
+
+def get_system_user_list():
+    """<comment-ja>
+    登録されているシステムユーザリストを/etc/passwd形式で取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    info = []
+    _getent = '/usr/bin/getent'
+    command_args = [_getent,'passwd']
+
+    old_lang = os.environ['LANG']
+    os.environ['LANG'] = 'C'
+    (ret,res) = execute_command(command_args)
+    os.environ['LANG'] = old_lang
+
+    if ret != 0:
+        return info
+
+    for user in res:
+        info.append(user.split(':'))
+    return info
+
+def get_system_group_list():
+    """<comment-ja>
+    登録されているシステムグループリストを/etc/group形式で取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    info = []
+    _getent = '/usr/bin/getent'
+    command_args = [_getent,'group']
+
+    old_lang = os.environ['LANG']
+    os.environ['LANG'] = 'C'
+    (ret,res) = execute_command(command_args)
+    os.environ['LANG'] = old_lang
+
+    if ret != 0:
+        return {}
+
+    for group in res:
+        info.append(group.split(':'))
+    return info
+
+def str_repeat(string="",count=1):
+    """<comment-ja>
+    文字列を繰り返す
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    retval = ""
+    for _cnt in range(0,count):
+       retval = "%s%s" % (retval,string,)
+    return retval
+
+def _php_array_to_python_dict(string=""):
+    """<comment-ja>
+    PHPのvar_export形式の文字列をpythonの辞書文字列に変換する
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    lines = string.split("\n")
+
+    array_depth = 0
+
+    regex_array_s = " *array *\("
+    regex_array_e = " *\)(?P<comma>,?)"
+    regex_list    = " *[0-9]+ *=> *(?P<value>.+)"
+    regex_dict    = " *(?P<key>'.+') *=> *"
+
+    array_types = []
+    _b_array = False
+    new_lines = []
+    for _aline in lines:
+        indent = ""
+        if array_depth > 0:
+            indent = str_repeat("   ",array_depth-1)
+        m_array_s = re.match(regex_array_s,_aline.rstrip())
+        m_array_e = re.match(regex_array_e,_aline.rstrip())
+        m_list    = re.match(regex_list   ,_aline)
+        m_dict    = re.match(regex_dict   ,_aline)
+        if m_array_s is not None:
+            array_depth = array_depth + 1
+            _b_array = True
+            continue
+        elif m_array_e is not None:
+            array_depth = array_depth - 1
+            array_type = array_types.pop()
+            if array_type == "list":
+                new_aline = "%s]%s" % (indent,m_array_e.group('comma'),)
+            if array_type == "dict":
+                new_aline = "%s}%s" % (indent,m_array_e.group('comma'),)
+            _b_array = False
+        elif m_list is not None:
+            if _b_array is True:
+                array_types.append("list")
+                new_aline = "%s[" % indent
+                new_lines.append(new_aline)
+            value = m_list.group('value')
+            new_aline = "%s%s" % (indent,re.sub("'","\'",value),)
+            _b_array = False
+        elif m_dict is not None:
+            if _b_array is True:
+                array_types.append("dict")
+                new_aline = "%s{" % indent
+                new_lines.append(new_aline)
+            key = m_dict.group('key')
+            value = re.sub(regex_dict,"%s:" % key, _aline)
+            new_aline = "%s%s" % (indent,value,)
+            _b_array = False
+        else:
+            new_aline = "%s%s" % (indent,_aline,)
+            _b_array = False
+
+        new_aline = re.sub("':false,","':False,", new_aline)
+        new_aline = re.sub("':true," ,"':True," , new_aline)
+        new_lines.append(new_aline)
+
+    #print "\n".join(new_lines)
+    return "\n".join(new_lines)
+ 
+def python_dict_to_php_array(dictionary={},var_name=""):
+    """<comment-ja>
+    pythonの辞書配列をPHPの連想配列(できればvar_export形式)の文字列に変換する
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    import signal
+
+    dict_str = preprint_r(dictionary,return_var=True)
+    dict_str = dict_str.replace('{','array(')
+    dict_str = dict_str.replace('\':','\' =>')
+    dict_str = dict_str.replace('}',')')
+    dict_str = dict_str.replace('=> [','=> array(')
+    dict_str = dict_str.replace('])','))')
+    dict_str = re.sub("\)$",')',dict_str)
+
+    php_start = "<?php\n"
+    php_end   = "?>\n"
+
+    # Convert var_export format
+    _php = "/usr/bin/php"
+    if is_executable(_php):
+        _script = "\"<?php var_export(%s); ?>\"" % (re.sub("[\r\n]"," ",dict_str),)
+        signal.alarm(10)
+        proc = subprocess.Popen(_php,
+                   bufsize=1,
+                   shell=True,
+                   stdin=subprocess.PIPE,
+                   stdout=subprocess.PIPE,
+                   stderr=subprocess.PIPE)
+
+        proc.stdin.write(_script)
+        output = proc.communicate()
+        ret = proc.wait()
+        signal.alarm(0)
+        if ret == 0:
+            dict_str = "".join(output)
+            dict_str = re.sub("^\"","",dict_str)
+            dict_str = re.sub("\"$","",dict_str)
+
+    header = php_start;
+    if var_name != "":
+        header = "%s$%s = " % (header,var_name,)
+    footer = php_end
+
+    return "%s%s;\n%s" % (header,dict_str,footer,)
+
+def php_array_to_python_dict(string=""):
+    """<comment-ja>
+    PHPの連想配列(var_export形式)の文字列をpythonの辞書配列に変換する
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    retval = False
+
+    string = re.sub("^<[\?%](php)?"         ,'' ,string.lstrip())
+    string = re.sub("[\?%]>$"               ,'' ,string.rstrip())
+    string = re.sub("^\\$[a-zA-Z0-9_]+ *= *",'' ,string.lstrip())
+    dict_str = string
+    dict_str = dict_str.replace('array('    ,'{')
+    dict_str = dict_str.replace('\' =>'     ,'\':')
+    dict_str = dict_str.replace(')'         ,'}')
+    dict_str = dict_str.replace('=> array(' ,'=> [')
+    dict_str = dict_str.replace('))'        ,'])')
+    dict_str = re.sub("{'(.*':.*)'}"         ,"['\\1']" ,dict_str)
+    dict_str = re.sub("\);"                   ,')',dict_str)
+    try:
+        exec("retval = %s" % dict_str)
+    except:
+        try:
+            # Read by var_export format
+            exec("retval = %s" % _php_array_to_python_dict(string))
+        except:
+            raise
+
+    return retval
+
+def get_karesansui_version():
+    import karesansui
+    return karesansui.__version__ + '.' + karesansui.__release__
+
+def get_blocks(d='/dev'):
+    """<comment-ja>
+    指定したフォルダに存在するブロックデバイス名一覧を取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    ret = []
+    for f in os.listdir(d):
+        if stat.S_ISBLK(os.stat("%s/%s" % (d, f))[stat.ST_MODE]) is True:
+            ret.append(f)
+
+    return set(ret)
+
+def get_hdd_list(_prefix='/dev'):
+    """<comment-ja>
+    OSで認識しているハードディスク名一覧を取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    from karesansui.lib.const import HDD_TYPES_REGEX
+    blocks = get_blocks(_prefix)
+    ret = []
+    for _type in HDD_TYPES_REGEX:
+        _regex = re.compile(_type)
+        for _b in blocks:
+            if _regex.match(_b):
+                ret.append(_b)
+    return ret
+
+def get_fs_info():
+    """<comment-ja>
+    OSで認識しているファイルシステムの情報(dfコマンドの結果)を取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    ret = []
+    tmp_dict = {}
+    pipe = os.popen("LANG=C /bin/df -m -P")
+    try:
+        data = []
+        for line in pipe.readlines():
+            data.append(re.sub(r'[ \t]', ' ', line).split())
+    finally:
+        pipe.close()
+
+    for i in range(1, len(data)):
+        for j in range(0,6):
+            tmp_dict[data[0][j]] = data[i][j]
+
+        ret.append(tmp_dict)
+        tmp_dict = {}
+
+    return ret
+
+def read_file(filepath):
+    """<comment-ja>
+    指定されたファイルの中身を取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    ret = ""
+    try:
+        fp = open(filepath, "r")
+        fcntl.lockf(fp.fileno(), fcntl.LOCK_SH)
+        for line in fp.readlines():
+            ret = ret + line
+        fcntl.lockf(fp.fileno(), fcntl.LOCK_UN)
+        fp.close()
+    except:
+        ret = False
+
+    return ret
+
+def create_epochsec(year, month, day, hour=0, minute=0, second=0):
+    return str(int(time.mktime(datetime.datetime(year, month, day, hour, minute, second).timetuple())))
+
+def get_hostname():
+    import socket
+#   return socket.gethostname()
+    return socket.getfqdn()
+
+def host2ip(host):
+    import socket
+    return socket.gethostbyname(host)
+
+def locale_dummy(str):
+    return str
+
+def symlink2real(symlink):
+    """<comment-ja>
+    シンボリック先の実体ファイルのファイル名を取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    path = os.path.realpath(symlink)
+    filename = os.path.basename(path)
+    sfilename = filename.split(".")
+    if len(sfilename) < 2:
+        return (os.path.dirname(path), sfilename[0], "")
+    else:
+        name = ""
+        for x in xrange(len(sfilename)-1):
+            name = os.path.join(name, sfilename[x])
+        return (os.path.dirname(path), name, sfilename[len(sfilename)-1])
+
+def get_filelist(dir_path="/"):
+    """<comment-ja>
+    指定したディレクトリに存在するファイル名の一覧を取得します。
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    if os.path.isdir(dir_path):
+        filelist = os.listdir(dir_path)
+    else:
+        filelist = []
+
+    return filelist
+
+def get_bonding_info(name=None):
+    """
+    <comment-ja>
+    bondingデバイス情報を取得する
+
+    @param name: 取得したいデバイス名(省略時は全デバイス情報が指定されたとみなす) 「regex:bond0」のようにプレフィックスにregex:を付けると正規表現にマッチしたデバイス名の情報を全て取得できる。
+    @return:    デバイス情報が格納されたdict配列
+                配列の内容例
+                {'bond0': {  'mode'    : 'fault-tolerance (active-backup)',
+                             'primary' : 'eth0',
+                             'active'  : 'eth0',
+                             'slave'   : ['eth0', 'eth1'],
+                          }
+
+    </comment-ja>
+    <comment-en>
+    Get computer's bonding interface information
+
+    @param name: bonding device name
+    @return: a dict with: mode, slave etc...
+    @rtype: dict
+    </comment-en>
+    """
+    info = {}
+    _info = {}
+    _proc_bonding_dir = '/proc/net/bonding'
+
+    mode_regex = re.compile(r"""^Bonding Mode:\s*(?P<mode>.+)""")
+    primary_regex = re.compile(r"""^Primary Slave:\s*(?P<primary>eth[0-9]+)""")
+    active_regex = re.compile(r"""^Currently Active Slave:\s*(?P<active>eth[0-9]+)""")
+    slave_regex = re.compile(r"""^Slave Interface:\s*(?P<slave>eth[0-9]+)""")
+
+    for device in get_filelist(_proc_bonding_dir):
+        _info[device] = {}
+        _info[device]['slave'] = []
+        for aline in read_file("%s/%s" % (_proc_bonding_dir, device)).split("\n"):
+            if aline.strip() == "":
+                continue
+
+            m = mode_regex.match(aline)
+            if m:
+                _info[device]['mode'] = m.group('mode')
+
+            m = primary_regex.match(aline)
+            if m:
+                _info[device]['primary'] = m.group('primary')
+
+            m = active_regex.match(aline)
+            if m:
+                _info[device]['active'] = m.group('active')
+
+            m = slave_regex.match(aline)
+            if m:
+                _info[device]['slave'].append(m.group('slave'))
+
+    all_info = dict_ksort(_info)
+
+    if name == None:
+        return all_info
+
+    regex_regex = re.compile(r"""^regex:(?P<regex>.*)""")
+    m = regex_regex.match(name)
+
+    for dev,value in all_info.iteritems():
+        if m == None:
+            if dev == name:
+                info[dev] = value
+                return info
+        else:
+            regex = m.group('regex')
+            query_regex = re.compile(r""+regex+"")
+            n = query_regex.search(dev)
+            if n != None:
+                info[dev] = value
+    return info
+
+def get_bridge_info(name=None):
+    """
+    <comment-ja>
+    Bridgeデバイス情報を取得する
+
+    @param name: 取得したいデバイス名(省略時は全デバイス情報が指定されたとみなす)
+    @return:    デバイス情報が格納されたdict配列
+                配列の内容例
+                {'eth0': ['peth0'] }
+
+    </comment-ja>
+    <comment-en>
+    Get computer's bridge interface information
+
+    @param name: bridge device name
+    @return: a dict with: bridge name
+    @rtype: dict
+    </comment-en>
+    """
+    info = {}
+
+    _sys_bridge_dir_tpl = '/sys/class/net/%s/bridge'
+    _sys_brif_dir_tpl = '/sys/class/net/%s/brif'
+
+    if_list = get_ifconfig_info()
+
+    for dev in if_list:
+        if os.path.exists(_sys_bridge_dir_tpl % (dev)):
+            if os.path.exists(_sys_brif_dir_tpl % (dev)):
+                info[dev] = get_filelist(_sys_brif_dir_tpl % (dev))
+
+    if name is not None:
+        if name in info:
+            return info[name]
+
+    return info
+
+
+def get_pwd_info():
+    """
+    <comment-ja>
+    全てのユーザ情報を取得する
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    return pwd.getpwall()
+
+def get_grp_info():
+    """
+    <comment-ja>
+    全てのグループ情報を取得する
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    return grp.getgrall()
+
+def get_filesystem_info():
+    """
+    <comment-ja>
+    使用できるファイルシステムを取得する
+    </comment-ja>
+    <comment-en>
+    TODO: English Comment
+    </comment-en>
+    """
+    ret = []
+    _filesystem_path = "/etc/filesystems"
+    nodev_regex = re.compile("^nodev")
+
+    data = read_file(_filesystem_path)
+    if data:
+        for line in data.split("\n"):
+            line = line.strip()
+            if line == "":
+                continue
+            if nodev_regex.match(line):
+                continue
+
+            ret.append(line)
+
+    return ret
+
+if __name__ == '__main__':
+    pass
